@@ -1,18 +1,23 @@
 from pytorch_lightning.callbacks.base import Callback
 from pytorch_lightning.callbacks.early_stopping import EarlyStopping
+from pytorch_lightning.callbacks.gpu_stats_monitor import GPUStatsMonitor
+from pytorch_lightning.callbacks.gpu_usage_logger import GpuUsageLogger
 from pytorch_lightning.callbacks.gradient_accumulation_scheduler import GradientAccumulationScheduler
 from pytorch_lightning.callbacks.lr_logger import LearningRateLogger
+from pytorch_lightning.callbacks.lr_monitor import LearningRateMonitor
 from pytorch_lightning.callbacks.model_checkpoint import ModelCheckpoint
-from pytorch_lightning.callbacks.progress import ProgressBarBase, ProgressBar
-from pytorch_lightning.callbacks.gpu_usage_logger import GpuUsageLogger
+from pytorch_lightning.callbacks.progress import ProgressBar, ProgressBarBase
+
 
 __all__ = [
     'Callback',
     'EarlyStopping',
-    'ModelCheckpoint',
+    'GPUStatsMonitor',
+    'GpuUsageLogger',
     'GradientAccumulationScheduler',
     'LearningRateLogger',
-    'ProgressBarBase',
+    'LearningRateMonitor',
+    'ModelCheckpoint',
     'ProgressBar',
-    'GpuUsageLogger'
+    'ProgressBarBase',
 ]
