@@ -23,6 +23,8 @@ import abc
 class Callback(abc.ABC):
     r"""
     Abstract base class used to build new callbacks.
+
+    Subclass this class and override any of the relevant hooks
     """
 
     def setup(self, trainer, pl_module, stage: str):
@@ -155,3 +157,12 @@ class Callback(abc.ABC):
 
     def on_keyboard_interrupt(self, trainer, pl_module):
         """Called when the training is interrupted by KeyboardInterrupt."""
+        pass
+
+    def on_save_checkpoint(self, trainer, pl_module):
+        """Called when saving a model checkpoint, use to persist state."""
+        pass
+
+    def on_load_checkpoint(self, checkpointed_state):
+        """Called when loading a model checkpoint, use to reload state."""
+        pass
