@@ -14,13 +14,13 @@
 
 import numbers
 from copy import copy
-from typing import Optional, Dict, Union, Sequence, Callable, MutableMapping, Any, List, Tuple
+from typing import Optional, Dict, Union, Sequence, Callable, MutableMapping, Any, List, Tuple, Iterable
 
 import torch
 from torch import Tensor
 import os
 
-from pytorch_lightning.metrics.converters import _sync_ddp_if_available
+from pytorch_lightning.utilities.distributed import sync_ddp_if_available
 
 
 class Result(Dict):
@@ -57,7 +57,7 @@ class Result(Dict):
         try:
             return super().__getitem__(key)
         except KeyError:
-            return super().__getitem__(f'step_{key}')
+            return super().__getitem__(f'{key}_step')
 
     def __getattr__(self, key: str) -> Any:
         try:
@@ -124,16 +124,19 @@ class Result(Dict):
 
         # sync across ddp
         if sync_dist and isinstance(value, (torch.Tensor, numbers.Number)):
-            value = _sync_ddp_if_available(value, group=sync_dist_group, reduce_op=sync_dist_op)
+            value = sync_ddp_if_available(value, group=sync_dist_group, reduce_op=sync_dist_op)
 
         if 'meta' not in self:
             self.__setitem__('meta', {})
 
         # if user requests both step and epoch, then we split the metric in two automatically
         # one will be logged per step. the other per epoch
+        was_forked = False
         if on_step and on_epoch:
+            was_forked = True
+
             # set step version
-            step_name = f'step_{name}'
+            step_name = f'{name}_step'
             self.__set_meta(
                 step_name,
                 value,
@@ -144,11 +147,12 @@ class Result(Dict):
                 reduce_fx=reduce_fx,
                 tbptt_reduce_fx=tbptt_reduce_fx,
                 tbptt_pad_token=tbptt_pad_token,
+                forked=False
             )
             self.__setitem__(step_name, value)
 
             # set epoch version
-            epoch_name = f'epoch_{name}'
+            epoch_name = f'{name}_epoch'
             self.__set_meta(
                 epoch_name,
                 value,
@@ -159,23 +163,26 @@ class Result(Dict):
                 reduce_fx=reduce_fx,
                 tbptt_reduce_fx=tbptt_reduce_fx,
                 tbptt_pad_token=tbptt_pad_token,
+                forked=False
             )
             self.__setitem__(epoch_name, value)
-        else:
-            self.__set_meta(
-                name,
-                value,
-                prog_bar,
-                logger,
-                on_step,
-                on_epoch,
-                reduce_fx,
-                tbptt_reduce_fx=tbptt_reduce_fx,
-                tbptt_pad_token=tbptt_pad_token,
-            )
 
-            # set the value
-            self.__setitem__(name, value)
+        # always log the original metric
+        self.__set_meta(
+            name,
+            value,
+            prog_bar,
+            logger,
+            on_step,
+            on_epoch,
+            reduce_fx,
+            tbptt_reduce_fx=tbptt_reduce_fx,
+            tbptt_pad_token=tbptt_pad_token,
+            forked=was_forked
+        )
+
+        # set the value
+        self.__setitem__(name, value)
 
     def __set_meta(
         self,
@@ -188,6 +195,7 @@ class Result(Dict):
         reduce_fx: Callable,
         tbptt_pad_token: int,
         tbptt_reduce_fx: Callable,
+        forked: bool
     ):
         # set the meta for the item
         meta_value = value
@@ -200,6 +208,7 @@ class Result(Dict):
             value=meta_value,
             tbptt_reduce_fx=tbptt_reduce_fx,
             tbptt_pad_token=tbptt_pad_token,
+            forked=forked
         )
 
         self['meta'][name] = meta
@@ -208,7 +217,12 @@ class Result(Dict):
         _internal = self['meta']['_internal']
         _internal['_reduce_on_epoch'] = max(_internal['_reduce_on_epoch'], on_epoch)
 
-    def track_batch_size(self, batch_size):
+    def track_batch_size(self, batch):
+        try:
+            batch_size = self.unpack_batch_size(batch)
+        except RecursionError as re:
+            batch_size = 1
+
         meta = self['meta']
         meta['_internal']['batch_sizes'].append(batch_size)
 
@@ -221,9 +235,10 @@ class Result(Dict):
 
         return result
 
-    def get_batch_log_metrics(self) -> dict:
+    def get_batch_log_metrics(self, include_forked_originals=True) -> dict:
         """
         Gets the metrics to log at the end of the batch step
+
         """
         result = {}
 
@@ -231,6 +246,10 @@ class Result(Dict):
         for k, options in meta.items():
             if k == '_internal':
                 continue
+
+            if options['forked'] and not include_forked_originals:
+                continue
+
             if options['logger'] and options['on_step']:
                 result[k] = self[k]
         return result
@@ -263,7 +282,7 @@ class Result(Dict):
                 result[k] = self[k]
         return result
 
-    def get_batch_pbar_metrics(self):
+    def get_batch_pbar_metrics(self, include_forked_originals=True):
         """
         Gets the metrics to log at the end of the batch step
         """
@@ -273,6 +292,9 @@ class Result(Dict):
         for k, options in meta.items():
             if k == '_internal':
                 continue
+            if options['forked'] and not include_forked_originals:
+                continue
+
             if options['prog_bar'] and options['on_step']:
                 result[k] = self[k]
         return result
@@ -299,9 +321,30 @@ class Result(Dict):
     def __copy__(self):
         newone = type(self)()
         for k, v in self.items():
+            if isinstance(v, torch.Tensor):
+                v = v.detach()
             newone[k] = copy(v)
         return newone
 
+    def unpack_batch_size(self, sample):
+        """
+        Recursively unpack sample to find a torch.Tensor.
+        returns len(tensor) when found, or 1 when it hits an empty or non iterable.
+        """
+        if isinstance(sample, torch.Tensor):
+            size = sample.size(0)
+        elif isinstance(sample, str):
+            return len(sample)
+        elif isinstance(sample, dict):
+            sample = next(iter(sample.values()), 1)
+            size = self.unpack_batch_size(sample)
+        elif isinstance(sample, Iterable):
+            sample = next(iter(sample), 1)
+            size = self.unpack_batch_size(sample)
+        else:
+            size = 1
+        return size
+
     @classmethod
     def gather(cls, outputs):
         meta = outputs[0].get('meta')
@@ -349,9 +392,14 @@ class Result(Dict):
     @classmethod
     def reduce_on_epoch_end(cls, outputs):
         # get the batch sizes for all outputs
-        batch_sizes = torch.stack([x.get_batch_sizes() for x in outputs]).view(-1)
+        batch_sizes = []
+        meta = {}
+        for x in outputs:
+            batch_sizes.append(x.get_batch_sizes())
+            meta.update(x['meta'])
+
+        batch_sizes = torch.stack(batch_sizes).view(-1)
 
-        meta = outputs[0]['meta']
         result = cls()
         result = recursive_gather(outputs, result)
         recursive_stack(result)
@@ -363,11 +411,16 @@ class Result(Dict):
             if option['on_epoch']:
                 fx = option['reduce_fx']
                 if fx == torch.mean:
-                    reduced_val = weighted_mean(result[k], batch_sizes)
+                    try:
+                        reduced_val = weighted_mean(result[k], batch_sizes)
+                    except Exception as e:
+                        reduced_val = torch.mean(result[k])
                 else:
                     reduced_val = fx(result[k])
 
                 result[k] = reduced_val
+            else:
+                del result[k]
 
         result['meta'] = meta
         return result
@@ -376,12 +429,17 @@ class Result(Dict):
     def reduce_across_time(cls, time_outputs):
         # auto-reduce across time for tbptt
         meta = time_outputs[0]['meta']
+
+        # in 1.0 the results have 'extra'. Once we deprecate 0.10.0 we may not need this
+        if 'extra' in time_outputs[0]:
+            [x.pop('extra', None) for x in time_outputs]
+
         result = cls()
         result = recursive_gather(time_outputs, result)
         recursive_stack(result)
 
         for k, value in result.items():
-            if k == 'meta':
+            if k in ['meta', 'extra']:
                 continue
 
             # pick the reduce fx
@@ -389,7 +447,12 @@ class Result(Dict):
                 tbptt_reduce_fx = torch.mean
             else:
                 tbptt_reduce_fx = meta[k]['tbptt_reduce_fx']
-            result[k] = tbptt_reduce_fx(value)
+
+            if isinstance(value, dict):
+                # TODO: recursive reduce:
+                _recursive_fx_apply(value, tbptt_reduce_fx)
+            else:
+                result[k] = tbptt_reduce_fx(value.float())
 
         result['meta'] = meta
         return result
@@ -435,12 +498,14 @@ def recursive_gather(outputs: Sequence[dict], result: Optional[MutableMapping] =
 
         for k, v in out.items():
             if isinstance(v, dict):
-                v = recursive_gather([v], result)
+                in_d = result.get(k, {})
+                v = recursive_gather([v], in_d)
+                result[k] = v
+            else:
+                if k not in result:
+                    result[k] = []
 
-            if k not in result:
-                result[k] = []
-
-            result[k].append(v)
+                result[k].append(v)
 
     return result
 
@@ -453,6 +518,18 @@ def recursive_stack(result: MutableMapping):
         result[k] = collate_tensors(v)
 
 
+def _recursive_fx_apply(input: dict, fx):
+    for k, v in input.items():
+        if isinstance(v, list):
+            v = torch.tensor(v)
+
+        if isinstance(v, torch.Tensor):
+            v = fx(v.float())
+            input[k] = v
+        else:
+            _recursive_fx_apply(v, fx)
+
+
 def collate_tensors(items: Union[List, Tuple]) -> Union[Tensor, List, Tuple]:
     if not items or not isinstance(items, (list, tuple)) or any(not isinstance(item, Tensor) for item in items):
         # items is not a sequence, empty, or contains non-tensors
@@ -497,8 +574,18 @@ class TrainResult(Result):
                 return result
 
         Args:
-            early_stop_on:
-            checkpoint_on:
+            minimize: Metric currently being minimized.
+            early_stop_on: Metric to early stop on.
+                Should be a one element tensor if combined with default
+                :class:`~pytorch_lightning.callbacks.early_stopping.EarlyStopping`.
+                If this result is returned by
+                :meth:`~pytorch_lightning.core.lightning.LightningModule.training_step`,
+                the specified value will be averaged across all steps.
+            checkpoint_on: Metric to checkpoint on.
+                Should be a one element tensor if combined with default checkpoint callback.
+                If this result is returned by
+                :meth:`~pytorch_lightning.core.lightning.LightningModule.training_step`,
+                the specified value will be averaged across all steps.
             hiddens:
         """
 
@@ -652,8 +739,17 @@ class EvalResult(Result):
                 return result
 
         Args:
-            early_stop_on:
-            checkpoint_on:
+            early_stop_on: Metric to early stop on.
+                Should be a one element tensor if combined with default
+                :class:`~pytorch_lightning.callbacks.early_stopping.EarlyStopping`.
+                If this result is returned by
+                :meth:`~pytorch_lightning.core.lightning.LightningModule.validation_step`,
+                the specified value will be averaged across all steps.
+            checkpoint_on: Metric to checkpoint on.
+                Should be a one element tensor if combined with default checkpoint callback.
+                If this result is returned by
+                :meth:`~pytorch_lightning.core.lightning.LightningModule.validation_step`,
+                the specified value will be averaged across all steps.
             hiddens:
         """
 
@@ -780,8 +876,11 @@ class EvalResult(Result):
             )
 
     def get_callback_metrics(self) -> dict:
-        result = {'val_early_stop_on': self.early_stop_on, 'val_checkpoint_on': self.checkpoint_on}
-
+        result = {}
+        if self.early_stop_on:
+            result['early_stop_on'] = self.early_stop_on
+        if self.checkpoint_on:
+            result['checkpoint_on'] = self.checkpoint_on
         return result
 
     def write(self, name: str, values: Union[Tensor, list], filename: str = 'predictions.pt'):
@@ -841,7 +940,41 @@ class EvalResult(Result):
 
 
 def weighted_mean(result, weights):
-    weights = weights.to(result.device)
-    numerator = torch.dot(result.float(), weights.t().float())
-    result = numerator / weights.sum().float()
+
+    if isinstance(result, dict):
+        _process_dataloader_aggregated_steps(result, weights)
+    else:
+        if isinstance(result, list):
+            result = torch.tensor(result)
+
+        weights = weights.to(result.device)[:result.size(0)]
+        numerator = torch.dot(result.float(), weights.transpose(-1, 0).float())
+        result = numerator / weights.sum().float()
     return result
+
+
+def _process_dataloader_aggregated_steps(result, weights):
+    internal_keys = {'meta'}
+
+    moved = False
+
+    for k, v in result.items():
+        if k in internal_keys:
+            continue
+
+        # make sure v is a tensor
+        if not isinstance(v, torch.Tensor):
+            v = torch.tensor(v)
+
+        # move to memory only once
+        if not moved:
+            weights = weights.to(v.device)
+            moved = True
+
+        # move weights to same device as value to reduce
+        weights_t = weights[:v.size(0)]
+
+        # weighted mean
+        numerator = torch.dot(v.float(), weights_t.transpose(-1, 0).float())
+        v = numerator / weights.sum().float()
+        result[k] = v
