@@ -45,8 +45,8 @@ This is the basic use of the trainer:
 
 --------
 
-Trainer in Python scrips
-------------------------
+Trainer in Python scripts
+-------------------------
 In Python scripts, it's recommended you use a main function to call the Trainer.
 
 .. code-block:: python
@@ -213,7 +213,7 @@ auto_scale_batch_size
 Automatically tries to find the largest batch size that fits into memory,
 before any training.
 
-.. testcode::
+.. code-block::
 
     # default used by the Trainer (no scaling of batch size)
     trainer = Trainer(auto_scale_batch_size=None)
@@ -221,6 +221,9 @@ before any training.
     # run batch size scaling, result overrides hparams.batch_size
     trainer = Trainer(auto_scale_batch_size='binsearch')
 
+    # call tune to find the batch size
+    trainer.tune(model)
+
 auto_select_gpus
 ^^^^^^^^^^^^^^^^
 
@@ -239,9 +242,9 @@ Example::
 auto_lr_find
 ^^^^^^^^^^^^
 Runs a learning rate finder algorithm (see this `paper <https://arxiv.org/abs/1506.01186>`_)
-before any training, to find optimal initial learning rate.
+when calling trainer.tune(), to find optimal initial learning rate.
 
-.. testcode::
+.. code-block:: python
 
     # default used by the Trainer (no learning rate finder)
     trainer = Trainer(auto_lr_find=False)
@@ -251,11 +254,19 @@ Example::
     # run learning rate finder, results override hparams.learning_rate
     trainer = Trainer(auto_lr_find=True)
 
+    # call tune to find the lr
+    trainer.tune(model)
+
+Example::
+
     # run learning rate finder, results override hparams.my_lr_arg
     trainer = Trainer(auto_lr_find='my_lr_arg')
 
+    # call tune to find the lr
+    trainer.tune(model)
+
 .. note::
-    See the `learning rate finder guide <lr_finder.rst>`_
+    See the :ref:`learning rate finder guide <lr_finder>`.
 
 benchmark
 ^^^^^^^^^
@@ -293,9 +304,9 @@ callbacks
 ^^^^^^^^^
 
 Add a list of user defined callbacks. These callbacks DO NOT replace the explicit callbacks
-(loggers, EarlyStopping or ModelCheckpoint).
+(loggers or ModelCheckpoint).
 
-.. note:: Only user defined callbacks (ie: Not EarlyStopping or ModelCheckpoint)
+.. note:: Only user defined callbacks (ie: Not ModelCheckpoint)
 
 .. code-block:: python
 
@@ -344,11 +355,37 @@ Example::
         filepath=os.getcwd(),
         save_top_k=True,
         verbose=True,
-        monitor='val_loss',
+        monitor='checkpoint_on',
         mode='min',
         prefix=''
     )
 
+cluster_environment
+^^^^^^^^^^^^^^^^^^^
+Environment to connect arbitrary cluster backends. Lightning automatically handles:
+
+- SLURM
+- TorchElastic
+
+For any other non-supported cluster environment, define your own class and pass it in.
+
+.. code-block:: python
+
+    from pytorch_lightning.cluster_environments import cluster_environment
+
+    class MyCluster(ClusterEnvironment):
+
+        def master_address(self):
+            return your_master_address
+
+        def master_port(self):
+            return your_master_port
+
+        def world_size(self):
+            return the_world_size
+
+    trainer = Trainer(cluster_environment=cluster_environment())
+
 default_root_dir
 ^^^^^^^^^^^^^^^^
 
@@ -396,36 +433,16 @@ Example::
 .. note:: this option does not apply to TPU. TPUs use ```ddp``` by default (over each core)
 
 See Also:
-    - `Multi-GPU training guide <multi_gpu.rst>`_
-    - `Multi-node (SLURM) guide <slurm.rst>`_
+    - :ref:`Multi-GPU training guide <multi_gpu>`.
+    - :ref:`Multi-node (SLURM) guide <slurm>`.
 
 early_stop_callback
 ^^^^^^^^^^^^^^^^^^^
 
-Callback for early stopping.
-early_stop_callback (:class:`pytorch_lightning.callbacks.EarlyStopping`)
+.. warning:: .. deprecated:: 0.10.0.
 
-- ``True``: A default callback monitoring ``'val_loss'`` is created.
-   Will raise an error if ``'val_loss'`` is not found.
-- ``False``: Early stopping will be disabled.
-- ``None``: The default callback monitoring ``'val_loss'`` is created.
-- Default: ``None``.
-
-.. testcode::
-
-    from pytorch_lightning.callbacks import EarlyStopping
-
-    # default used by the Trainer
-    early_stop = EarlyStopping(
-        monitor='val_loss',
-        patience=3,
-        strict=False,
-        verbose=False,
-        mode='min'
-    )
-    trainer = Trainer(early_stop_callback=early_stop)
-
-.. note:: If ``'val_loss'`` is not found will work as if early stopping is disabled.
+    Deprecated since v0.10.0 and will be removed in v1.0. Configure the EarlyStopping callback class
+    and add it to the list of callbacks: ``Trainer(callbacks=[EarlyStopping(...)])``
 
 fast_dev_run
 ^^^^^^^^^^^^
@@ -493,7 +510,7 @@ Example::
     trainer = Trainer(gpus=[1, 4], num_nodes=4)
 
 See Also:
-    - `Multi-GPU training guide <multi_gpu.rst>`_
+    - :ref:`Multi-GPU training guide <multi_gpu>`.
 
 gradient_clip_val
 ^^^^^^^^^^^^^^^^^
@@ -565,20 +582,23 @@ Options:
 
 .. note:: Might slow performance because it uses the output of nvidia-smi.
 
-log_save_interval
-^^^^^^^^^^^^^^^^^
+flush_logs_every_n_steps
+^^^^^^^^^^^^^^^^^^^^^^^^
 
 Writes logs to disk this often.
 
 .. testcode::
 
     # default used by the Trainer
-    trainer = Trainer(log_save_interval=100)
+    trainer = Trainer(flush_logs_every_n_steps=100)
+
+See Also:
+    - :ref:`logging`
 
 logger
 ^^^^^^
 
-`Logger <loggers.rst>`_ (or iterable collection of loggers) for experiment tracking.
+:ref:`Logger <loggers>` (or iterable collection of loggers) for experiment tracking.
 
 .. testcode::
 
@@ -814,7 +834,7 @@ profiler
 ^^^^^^^^
 To profile individual steps during training and assist in identifying bottlenecks.
 
-See the `profiler documentation <profiler.rst>`_. for more details.
+See the :ref:`profiler documentation <profiler>`. for more details.
 
 .. testcode::
 
@@ -897,15 +917,19 @@ To resume training from a specific checkpoint pass in the path here.
     # resume from a specific checkpoint
     trainer = Trainer(resume_from_checkpoint='some/path/to/my_checkpoint.ckpt')
 
-row_log_interval
-^^^^^^^^^^^^^^^^
+log_every_n_steps
+^^^^^^^^^^^^^^^^^
 
 How often to add logging rows (does not write to disk)
 
 .. testcode::
 
     # default used by the Trainer
-    trainer = Trainer(row_log_interval=50)
+    trainer = Trainer(log_every_n_steps=50)
+
+See Also:
+    - :ref:`logging`
+
 
 sync_batchnorm
 ^^^^^^^^^^^^^^
@@ -916,21 +940,6 @@ Enable synchronization between batchnorm layers across all GPUs.
 
     trainer = Trainer(sync_batchnorm=True)
 
-val_percent_check
-^^^^^^^^^^^^^^^^^
-
-.. warning:: deprecated in v0.8.0 please use `limit_val_batches`. Will remove in 0.10.0
-
-test_percent_check
-^^^^^^^^^^^^^^^^^^
-
-.. warning:: deprecated in v0.8.0 please use `limit_test_batches`. Will remove in 0.10.0
-
-train_percent_check
-^^^^^^^^^^^^^^^^^^^
-
-.. warning:: deprecated in v0.8.0 please use `limit_train_batches`. Will remove in 0.10.0
-
 track_grad_norm
 ^^^^^^^^^^^^^^^
 
@@ -1093,4 +1102,4 @@ Trainer class API
 from pytorch_lightning.trainer.trainer import Trainer
 from pytorch_lightning.utilities.seed import seed_everything
 
-__all__ = ['Trainer', 'seed_everything']
+__all__ = ["Trainer", "seed_everything"]
