@@ -13,6 +13,7 @@
 # limitations under the License.
 
 from abc import ABC
+from copy import deepcopy
 from typing import Callable, List
 
 from pytorch_lightning.callbacks import Callback
@@ -45,10 +46,10 @@ class TrainerCallbackHookMixin(ABC):
         for callback in self.callbacks:
             callback.on_init_end(self)
 
-    def on_fit_start(self, model):
+    def on_fit_start(self):
         """Called when the trainer initialization begins, model has not yet been set."""
         for callback in self.callbacks:
-            callback.on_fit_start(self, model)
+            callback.on_fit_start(self, self.get_model())
 
     def on_fit_end(self):
         """Called when the trainer initialization begins, model has not yet been set."""
@@ -189,3 +190,22 @@ class TrainerCallbackHookMixin(ABC):
         """Called when the training is interrupted by KeyboardInterrupt."""
         for callback in self.callbacks:
             callback.on_keyboard_interrupt(self, self.get_model())
+
+    def on_save_checkpoint(self):
+        """Called when saving a model checkpoint."""
+        callback_states = {}
+        for callback in self.callbacks:
+            callback_class = type(callback)
+            state = callback.on_save_checkpoint(self, self.get_model())
+            if state:
+                callback_states[callback_class] = state
+        return callback_states
+
+    def on_load_checkpoint(self, checkpoint):
+        """Called when loading a model checkpoint."""
+        callback_states = checkpoint.get('callbacks')
+        for callback in self.callbacks:
+            state = callback_states.get(type(callback))
+            if state:
+                state = deepcopy(state)
+                callback.on_load_checkpoint(state)
