@@ -15,7 +15,6 @@
 import os
 import time
 from collections import Counter
-from enum import Enum
 from functools import wraps
 from typing import Callable, Any, Optional
 
@@ -39,7 +38,7 @@ class InternalDebugger(object):
 
     def __init__(self, trainer):
 
-        self.enabled = 'PL_DEV_DEBUG' in os.environ
+        self.enabled = os.environ.get('PL_DEV_DEBUG', '0') == '1'
         self.trainer = trainer
         self.logged_metrics = []
         self.pbar_added_metrics = []
@@ -50,6 +49,10 @@ class InternalDebugger(object):
         self.checkpoint_callback_history = []
         self.events = []
         self.saved_lr_scheduler_updates = []
+        self.train_dataloader_calls = []
+        self.val_dataloader_calls = []
+        self.test_dataloader_calls = []
+        self.dataloader_sequence_calls = []
 
     def track_event(
             self,
@@ -77,6 +80,36 @@ class InternalDebugger(object):
                 count += 1
         return count
 
+    @enabled_only
+    def track_load_dataloader_call(self, name, dataloaders):
+        loader_counts = len(dataloaders)
+
+        lengths = []
+        for dl in dataloaders:
+            try:
+                length = len(dl)
+            except Exception as e:
+                length = -1
+            lengths.append(length)
+
+        values = {
+            'global_step': self.trainer.global_step,
+            'epoch': self.trainer.current_epoch,
+            'num_loaders': loader_counts,
+            'lengths': lengths,
+            'name': name
+        }
+
+        # track the sequence in case we need to verify the sequence
+        self.dataloader_sequence_calls.append(values)
+
+        if 'train' in name:
+            self.train_dataloader_calls.append(values)
+        elif 'val' in name:
+            self.val_dataloader_calls.append(values)
+        elif 'test' in name:
+            self.test_dataloader_calls.append(values)
+
     @enabled_only
     def track_logged_metrics_history(self, scalar_metrics):
         scalar_metrics['global_step'] = self.trainer.global_step
@@ -121,15 +154,14 @@ class InternalDebugger(object):
         self.pbar_added_metrics.append(metrics)
 
     @enabled_only
-    def track_early_stopping_history(self, current):
-        es = self.trainer.early_stop_callback
+    def track_early_stopping_history(self, callback, current):
         debug_dict = {
             'epoch': self.trainer.current_epoch,
             'global_step': self.trainer.global_step,
             'rank': self.trainer.global_rank,
             'current': current,
-            'best': es.best_score,
-            'patience': es.wait_count
+            'best': callback.best_score,
+            'patience': callback.wait_count
         }
         self.early_stopping_history.append(debug_dict)
 
